<?php

/**
 *
 * coWiki - includes/cowiki/class/core/class.Registry.php
 *
 * $Id: class.Registry.php,v 1.5 2003/09/10 04:49:08 kschroed Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <cowiki-license@develnet.org> so we
 * can mail you a copy immediately.
 *
 * @package     core
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.5 $
 *
 */

/**
 * coWiki - Registry class
 *
 * @package     core
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class Registry extends Object {

    protected static
        $Instance = null;

    /**
     * Get instance
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function getInstance() {
        if (!self::$Instance) {
            self::$Instance = new Registry;
        }
        return self::$Instance;
    }

    /**
     * Class constructor
     *
     * @access  protected
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    protected function __construct() {
        $Context = RuntimeContext::getInstance();

        $sConf = getDirName(__FILE__).'../../core.conf';
        $aConfig = @parse_ini_file($sConf, true);

        if (!is_array($aConfig)) {
            $Context->addError(515);
            $Context->terminate();
        }

        foreach ($aConfig as $aSection => $aData) {
            foreach ($aData as $k => $v) {
                $this->set($aSection.'_'.$k, $v);
            }
        }

        // ----------------------------------------------------------------

        // Fetch template configuration

        $sPath = $Context->getEnvironment()->get('PHP_SELF');

        $sTpl = getRequestBasePath($sPath) . 'tpl/';

        // Concat path to active template
        $sActTpl = $sTpl . $this->get('RUNTIME_TEMPLATE_ACTIVE') . '/';

        // Concat path to default template (for fallback reasons)
        $sDefTpl = $sTpl . 'default/';

        // Basic template path
        $this->set('PATH_TEMPLATE', $sTpl);

        // Active template
        $this->set('PATH_TEMPLATE_ACTIVE', $sActTpl);

        // Path to active images
        $this->set('PATH_IMAGES', $sActTpl . 'img/');

        // Fallback template path
        $this->set('PATH_TEMPLATE_DEFAULT', $sDefTpl);

        // ----------------------------------------------------------------

        // Get template configuration file and define constants
        $this->getTplConf($sActTpl);
    }

    // ----------------------------------------------------------------

    /**
     * Get template configuration file and define constants
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    public function getTplConf($sActTpl) {
        $Context = RuntimeContext::getInstance();

        $sIniPath = $Context->getEnvironment()->get('DOCUMENT_ROOT')
                    . $sActTpl . 'tpl.conf';

        $aConfig = @parse_ini_file($sIniPath, true);

        foreach ($aConfig as $aSection => $aData) {
            foreach ($aData as $k => $v) {
                $this->set($aSection.'_'.$k, $v);
            }
        }
    }

    /**
     * Overwrite parent method
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    public function has($sKey) {
        if (isset($this->__aBaseProp[$sKey])) {
            return true;
        }

        if (defined($sKey)) {
            return true;
        }

        return false;
    }

    /**
     * Remove
     *
     * @access  public
     * @param   string
     * @return  null
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function remove($sKey) {
        if (isset($this->__aBaseProp[$sKey])) {
            unset($this->__aBaseProp[$sKey]);
        }

        return null;
    }

    /**
     * Overwrite parent method
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    public function &get($sKey) {
        if (isset($this->__aBaseProp[$sKey])) {
            return $this->__aBaseProp[$sKey];
        }

        if (defined($sKey)) {
            $sValue = constant($sKey);
            return $sValue;
        }

        return null;
    }

} // of class

?>
