<?php

/**
 *
 * coWiki - includes/cowiki/class/dao/class.StorageMySQL.php
 *
 * $Id: class.StorageMySQL.php,v 1.14 2003/09/10 04:49:09 kschroed Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <cowiki-license@develnet.org> so we
 * can mail you a copy immediately.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.14 $
 *
 */

/**
 * coWiki - Storage MySQL class
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class StorageMySQL extends Object {

    protected
        $rLink = null,
        $rResult = null,
        $aLocked = array(),
        $sLastId = 0;

    /**
     * Class constructor
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    public function __construct() {
        if (!function_exists('mysql_connect')) {
            RuntimeContext::getInstance()->addError(540, 'MySQL');
            RuntimeContext::getInstance()->terminate();
        }
    }

    /**
     * Connect
     *
     * @access  public
     * @param   string
     * @param   string
     * @param   string
     * @param   string
     * @param   string
     * @return  object
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function connect($sHost, $sPort, $sUser, $sPass, $sDatabase) {
        $bSuccess = true;

        if ($sPort != '') {
            $sPort = ':' . $sPort;
        }

        if (!$this->rLink = @mysql_connect($sHost.$sPort, $sUser, $sPass)) {
            errorLogHandler(mysql_errno(),mysql_error(),__FILE__,__LINE__ );
            $bSuccess = false;
        }

        if(!@mysql_select_db($sDatabase, $this->rLink)) {
            errorLogHandler(mysql_errno(),mysql_error(),__FILE__,__LINE__ );
            $bSuccess = false;
        }

        return $bSuccess;
    }

    /**
     * Query
     *
     * @access  public
     * @param   string
     * @param   object
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$mQueryType"
     * @todo    [D11N]  Check return type
     */
    public function query($sQuery, $mQueryType = null) {
        //echo $sQuery.'<hr />'; // Debug output

        if (!$this->rResult = @mysql_query($sQuery, $this->rLink)) {
            errorLogHandler(mysql_errno(),mysql_error(),__FILE__,__LINE__ );

            RuntimeContext::getInstance()->addError(560, $mQueryType);
            RuntimeContext::getInstance()->terminate();
        }

        return $this->rResult;
    }

    /**
     * Fetch array
     *
     * @access  public
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nQueryId"
     * @todo    [D11N]  Check return type
     */
    public function fetchArray($nQueryId) {
        return @mysql_fetch_assoc($nQueryId);
    }

    /**
     * Fetch object
     *
     * @access  public
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nQueryId"
     */
    public function fetchObject($nQueryId) {
        $aData = $this->fetchArray($nQueryId);

        if (!is_array($aData)) {
            return null;
        }

        $aKeys = array_keys($aData);
        $Obj = new Object();

        for ($i=0, $n=sizeof($aKeys); $i<$n; $i++) {
            $Obj->set($aKeys[$i], $aData[$aKeys[$i]]);
        }

        return $Obj;
    }

    /**
     * Add limit to query
     *
     * @access  public
     * @param   string
     * @param   integer
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nFrom"
     * @todo    [D11N]  Check the parameter type of "$nCount"
     * @todo    [D11N]  Check return type
     */
    public function addLimitToQuery($sQuery, $nFrom, $nCount) {
        return $sQuery . ' LIMIT ' . $nFrom . ',' . $nCount;
    }

    /**
     * Get last insert id
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function getLastInsertId($sTableName) {
        return $this->sLastId;
    }

    /**
     * Escape string
     *
     * @access  private
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    private function escapeString($sStr) {
        return addslashes($sStr);
    }

    /**
     * Remove
     *
     * @access  public
     * @param   array
     * @return  ressource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function remove($aData) {
        $this->beginTransaction($aData['table']);

        $sQuery = 'DELETE FROM '.$aData['table'].' WHERE '.$aData['where'];
        $rResult = $this->query($sQuery, 'DELETE');

        $this->endTransaction($aData['table']);

        return $rResult;
    }

    /**
     * Insert
     *
     * @access  public
     * @param   array
     * @return  ressource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function insert($aData) {
        $this->beginTransaction($aData['table']);

        $sQuery = 'INSERT INTO ' . $aData['table'];
        $sValue = '';

        $sQuery .= ' (';
        $sValue .= ' VALUES (';

        while (list($key, $value) = each($aData['fields'])) {
            $sQuery .= $key;
            $sQuery .= ", ";
            $sValue .= "'";
            $sValue .= $this->escapeString($value);
            $sValue .= "', ";
        }

        // Remove last comma and space
        $sQuery = substr($sQuery, 0, -2);
        $sValue = substr($sValue, 0, -2);

        $sQuery .= ') ';
        $sValue .= ') ';

        $rResult = $this->query($sQuery . $sValue, 'INSERT');

        // Save the "last insert id", it would be falsified by the
        // "endTransaction()" method otherwise.
        $this->sLastId = @mysql_insert_id($this->rLink);

        $this->endTransaction($aData['table']);

        return $rResult;
    }

    /**
     * Update
     *
     * @access  public
     * @param   array
     * @return  ressource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function update($aData) {
        $this->beginTransaction($aData['table']);

        $sQuery = 'UPDATE ' . $aData['table'] . ' SET ';

        while (list($key, $value) = each($aData['fields'])) {
            $sQuery .= $key;
            $sQuery .= "='";
            $sQuery .= $this->escapeString($value);
            $sQuery .= "', ";
        }

        // Remove last comma and space
        $sQuery = substr($sQuery, 0, -2);
        $sQuery .= ' WHERE ' . $aData['where'];

        $rResult = $this->query($sQuery, 'UPDATE');

        $this->endTransaction($aData['table']);

        return $rResult;
    }

    /**
     * Free result
     *
     * @access  public
     * @param   ressource
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function freeResult($rResult) {
        @mysql_free_result($rResult);
    }

    /**
     * Begin transaction
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function beginTransaction($sTableName) {
        if (!isset($this->aLocked[$sTableName])) {
            $this->aLocked[$sTableName] = 0;
        }

        $this->aLocked[$sTableName]++;

        if ($this->aLocked[$sTableName] == 1) {
            $this->query('LOCK TABLES '.$sTableName.' WRITE', 'LOCK');
        }
    }

    /**
     * End transaction
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function endTransaction($sTableName) {
        if ($this->aLocked[$sTableName] != 0) {
            $this->aLocked[$sTableName]--;
        }

        if (array_sum($this->aLocked) == 0) {
            $this->query('UNLOCK TABLES', 'UNLOCK');
        }
    }

    /**
     * Provide an unique number - used to determine and avoid (multiple)
     * changes in a database record (this number is the actual
     * unix-timestamp plus a dash plus random eight cipher number)
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check return type
     */
    public function generateTan()  {
        mt_srand((double)microtime() * 1000000);
        return time() . '-' . mt_rand(10000000, 99999999);
    }

    /**
     * Get date time as string
     *
     * @access  public
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nStamp"
     * @todo    [D11N]  Check return type
     */
    public function getDateTimeAsString($nStamp) {
        return date('Y-m-d H:i:s', $nStamp);
    }

    /**
     * Capacity of fields containing textdata (in bytes)
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check return type
     */
    public function getTextCapacity() {
        return 65000;
    }

} // of class

/*
    With your feet on the air and your head on the ground
    Try this trick and spin it, yeah
    Your head will collapse if there's nothing in it
    And you'll ask yourself
    Where is my mind? Where is my mind? Where is my mind?

    Way out in the water, see it swimming

    I was swimming in the Caribbean
    Animals were hiding behind the rock
    Except for little fish
    When they told me east is west trying to talk to me, coy koi
    Where is my mind? Where is my mind? Where is my mind?

    Way out in the water, see it swimming

    With your feet on the air and your head on the ground
    Try this trick and spin it, yeah
    Your head will collapse if there's nothing in it
    And you'll ask yourself
    Where is my mind? Where is my mind? Where is my mind?

    Way out in the water, see it swimming

    With your feet on the air and your head on the ground
    Try this trick and spin it, yeah
*/

?>
