<?php

/**
 *
 * coWiki - includes/cowiki/class/io/class.FileInputStream.php
 *
 * $Id: class.FileInputStream.php,v 1.4 2003/09/10 04:49:10 kschroed Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <cowiki-license@develnet.org> so we
 * can mail you a copy immediately.
 *
 * @package     io
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.4 $
 *
 */

/**
 * coWiki - File input stream class
 *
 * @package     io
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class FileInputStream extends AbstractInputStream {

    protected
        $rFile     = null,
        $bOpener   = false,
        $sFileName = '';

    /**
     * Class constructor
     *
     * @access  public
     * @param   object
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check the parameter type of "$mIn"
     */
    public function __construct($mIn) {

        if (is_string($mIn)) {

            if (!is_readable($mIn)) {
                throw new Exception();
            }

            if (!($this->rFile = @fopen($mIn, 'rb'))) {
                throw new Exception();
            }

            // This object opened the resource
            $this->bOpener = true;
            $this->sFileName = $mIn;
        }

        if (is_resource($mIn)) {
           $this->rFile = $mIn;
        }
    }

    /**
     * Available
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function available() {
        if ($this->sFileName) {
            return @filesize($this->sFileName);
        }

        return false;
    }

    /**
     * &read
     *
     * @access  public
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nLen"
     */
    public function &read($nLen) {
        if (is_resource($this->rFile)) {
            return @fread($this->rFile, $nLen);
        }
        return false;
    }

    /**
     * &read all
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function &readAll() {
        $sBuf = '';

        if (is_resource($this->rFile)) {
            if ($this->sFileName) {
                $sReturn = fread($this->rFile, $this->available());
                return $sReturn;
            }

            while(!feof($this->rFile)) {
                $sBuf .= fread($this->rFile, 16384);
            }
        }

        return $sBuf;
    }

    /**
     * Skip
     *
     * @access  public
     * @param   integer
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nLen"
     */
    public function skip($nLen) {
        if (is_resource($this->rFile)) {
            @fseek($this->rFile, $nLen, SEEK_CUR);
        }
    }

    /**
     * Close
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function close() {
        if ($this->bOpener && is_resource($this->rFile)) {
            @fclose($this->rFile);
        }
        $this->rFile   = null;
        $this->bOpener = false;
    }

} // of class

?>
