<?php

/**
 *
 * coWiki - includes/cowiki/class/io/class.FileOutputStream.php
 *
 * $Id: class.FileOutputStream.php,v 1.4 2003/09/10 05:47:46 kschroed Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <cowiki-license@develnet.org> so we
 * can mail you a copy immediately.
 *
 * @package     io
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.4 $
 *
 */

/**
 * coWiki - File output stream class
 *
 * @package     io
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class FileOutputStream{

    protected
        $rFile   = null,
        $bOpener = false;

    /**
     * Class constructor
     *
     * @access  public
     * @param   object
     * @param   boolean
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check the parameter type of "$mOut"
     */
    public function __construct($mOut, $bAppend = false) {

        if (is_string($mOut)) {

            $sMode = $bAppend ? 'ab' : 'wb';

            if (!($this->rFile = @fopen($mOut, $sMode))) {
                throw new Exception();
            }

            if (!@flock($this->rFile, LOCK_EX)) {
                $this->close();
                throw new Exception();
            }

            // This object opened the resource
            $this->bOpener = true;
        }

        if (is_resource($mOut)) {
           $this->rFile = $mOut;
        }

    }

    /**
     * Write
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function write($sOut) {
        if (is_resource($this->rFile)) {
            if (!fwrite($this->rFile, $sOut)) {
                throw new Exception();
            }
        }
    }

    /**
     * Flush
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function flush() {
        if (is_resource($this->rFile)) {
            if(!fflush($this->rFile)) {
                throw new Exception();
            }
        }
    }

    /**
     * Close
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    public function close() {
        if ($this->bOpener && is_resource($this->rFile)) {
            @fclose($this->rFile);
        }
        $this->rFile   = null;
        $this->bOpener = false;
    }

} // of class

?>
