<?php

/**
 *
 * coWiki - includes/cowiki/core.base.php
 *
 * $Id: core.base.php,v 1.49 2003/10/20 19:24:57 dtg Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <cowiki-license@develnet.org> so we
 * can mail you a copy immediately.
 *
 * @package    core
 * @access     public
 *
 * @author     Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright  (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license    http://www.gnu.org/licenses/gpl.html
 * @version    $Revision: 1.49 $
 *
 */

    set_magic_quotes_runtime(0);

    header('Vary: Accept-Encoding, User-Agent');

    /**
     * Version information
     */
    include_once 'version.php';

    /**
     * Core functions
     */
    include_once 'core.function.php';

    // First read the uptodate english localized file.
    setCatalog('locale/', 'error');
    setCatalog('locale/', 'en.iso-8859-1');

    // --------------------------------------------------------------------

    define('CMD_CHUSR',      'chusr');
    define('CMD_NEWUSR',     'newusr');
    define('CMD_EDITUSR',    'editusr');
    define('CMD_PREFUSR',    'prefusr');
    define('CMD_NEWGRP',     'newgrp');
    define('CMD_EDITGRP',    'editgrp');
    define('CMD_NEWWEB',     'newweb');
    define('CMD_EDITWEB',    'editweb');
    define('CMD_SHOWDIR',    'showdir');
    define('CMD_NEWDIR',     'newdir');
    define('CMD_EDITDIR',    'editdir');
    define('CMD_NEWDOC',     'newdoc');
    define('CMD_EDITDOC',    'editdoc');
    define('CMD_SIMDOC',     'simdoc');
    define('CMD_AMBIDOC',    'ambidoc');
    define('CMD_MOVEDOC',    'movedoc');
    define('CMD_PRNTDOC',    'prntdoc');
    define('CMD_XMLDOC',     'xmldoc');
    define('CMD_SRCHDOC',    'srchdoc');
    define('CMD_SHOWHIST',   'showhist');
    define('CMD_COMPHIST',   'comphist');
    define('CMD_DIFFHIST',   'diffhist');
    define('CMD_RECOVHIST',  'recovhist');
    define('CMD_NEWCOM',     'newcom');
    define('CMD_LISTCOM',    'listcom');
    define('CMD_REPLYCOM',   'replycom');
    define('CMD_DETAILUSR',  'detusr');

    // --------------------------------------------------------------------

    include_once 'class/class.Object.php';
    include_once 'class/class.Exceptions.php';

    // --------------------------------------------------------------------

    // Register unserialize callback handler for objects coming from session
    ini_set('unserialize_callback_func', '__autoload');

    // Session settings and lift off
    ini_set('arg_separator.output', '&amp;');
    session_name('cowiki');
    @session_start();

    $Context  = RuntimeContext::getInstance();
    $Request  = $Context->getRequest();
    $Registry = $Context->getRegistry();

    // Load localized catalog
    if ($Registry->get('RUNTIME_LANGUAGE_LOCALE')) {
        if ($Registry->get('RUNTIME_LANGUAGE_LOCALE') != 'en.iso-8859-1') {
            setCatalog('locale/', $Registry->get('RUNTIME_LANGUAGE_LOCALE'));
        }
    }

    // --------------------------------------------------------------------

    // Check if we should teergrubing the client
    $aTeer = explode(',', $Registry->get('RUNTIME_TEERGRUBING_AGENT'));
    $sAgent = strtolower($Request->getAgent());

    for ($i=0, $n=sizeof($aTeer); $i<$n; $i++) {
        $sStr = strtolower(trim($aTeer[$i]));

        // Avoid user errors in teergrubing agent string
        if ($sStr == '') {
            continue;
        }

        if (substr_count($sAgent, $sStr)) {
            $nTimeOut = (int)$Registry->get('RUNTIME_TEERGRUBING_TIMEOUT');

            if ($nTimeOut < 0) {
                exit; // die
            }
            sleep($nTimeOut);
        }
    }

    // --------------------------------------------------------------------

    // Check if we should teergrubing the IP
    $aTeer = explode(',', $Registry->get('RUNTIME_TEERGRUBING_IP'));

    for ($i=0, $n=sizeof($aTeer); $i<$n; $i++) {
        $sStr = trim($aTeer[$i]);

        // Avoid user errors in teergrubing IP string
        if ($sStr == '') {
            continue;
        }

        if (preg_match('#^'.$sStr.'$#', $Request->getRemoteAddr())) {
            $nTimeOut = (int)$Registry->get('RUNTIME_TEERGRUBING_TIMEOUT');

            if ($nTimeOut < 0) {
                exit; // die
            }
            sleep($nTimeOut);
        }
    }

    // --------------------------------------------------------------------

    // Check if user is logged in ...
    $aTmp = $Context->getSessionVar('loginData', 'core');

    $bValid = is_array($aTmp)
              && isset($aTmp['userId'])
              && isset($aTmp['loggedIn'])
              && $aTmp['loggedIn'];

    if ($bValid) {
        $UserDAO = $Context->getUserDAO();
        $User = $UserDAO->getUserByUid($aTmp['userId']);

        if (is_object($User)) {
            $CurrUser = CurrentUser::getInstance();

            // Set properties in current user object
            $CurrUser->set('isValid', true);
            $CurrUser->set('userId',  $User->get('userId'));
            $CurrUser->set('groupId', $User->get('groupId'));
            $CurrUser->set('login',   $User->get('login'));
            $CurrUser->set('name',    $User->get('name'));
            $CurrUser->setMemberGroups($User->getMemberGroups());
        } else {
            // Force user reset
            $bValid = false;
        }
    }

    if (!$bValid) {
        $CurrUser = CurrentUser::getInstance();
        $CurrUser->reset();

        // Set localized name for the guest user
        $CurrUser->set('name', __('I18N_GUEST_FULLNAME'));
    }

    // Set current user object in context
    $Context->setCurrentUser($CurrUser);

    // --------------------------------------------------------------------

    // Set locales depending on loaded catalog file
    $aLocales = explode(',', __('I18N_LOCALES'));
    for ($i=0, $n=sizeof($aLocales); $i<$n; $i++) {
        if (@setlocale(LC_ALL, trim($aLocales[$i]))) {
            break;
        }
    }

    // --------------------------------------------------------------------

    // This & that ... before we start
    $Registry->set('ZEND_VERSION', zend_version());

    $Context->getReferrerDAO()->store($Request->getReferrer());

    $Registry->set('BASE_URI',    $Request->getHostUri());
    $Registry->set('REMOTE_ADDR', $Request->getRemoteAddr());
    $Registry->set('REMOTE_HOST', $Request->getRemoteHost());

    $Registry->set(
        'TIME',
        strftime(
            __('I18N_DATE_FORMAT').', '.__('I18N_TIME_FORMAT'),
            time()
        )
    );

    // --------------------------------------------------------------------

    ini_set('highlight.string',   $Registry->get('COLOR_CODE_STRING'));
    ini_set('highlight.comment',  $Registry->get('COLOR_CODE_COMMENT'));
    ini_set('highlight.keyword',  $Registry->get('COLOR_CODE_KEYWORD'));
    ini_set('highlight.bg',       $Registry->get('COLOR_CODE_BG'));
    ini_set('highlight.default',  $Registry->get('COLOR_CODE_DEFAULT'));
    ini_set('highlight.html',     $Registry->get('COLOR_CODE_HTML'));

    ob_start();     // ... at least let the fun start


    // === HELPER =========================================================

    /**
     * Class loader
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    function __autoload($sClass) {

        static $aPath = array(

            'authcallback' =>
            'auth/interface.AuthCallback',

            'authmanager' =>
            'auth/class.AuthManager',

            'authmysql' =>
            'auth/class.AuthMySQL',

            // ------------------------------------------------------------

            'composite' =>
            'core/interface.Composite',

            'compositeview' =>
            'core/interface.CompositeView',

            'compositeguardeditem' =>
            'core/class.CompositeGuardedItem',

            'utility' =>
            'core/class.Utility',

            'abstracttreeview' =>
            'core/class.AbstractTreeView',

            'simplecommenttreeview' =>
            'core/class.SimpleCommentTreeView',

            'debugtreeview' =>
            'core/class.DebugTreeView',

            'documentitem' =>
            'core/class.DocumentItem',

            'documentcontainer' =>
            'core/class.DocumentContainer',

            'user' =>
            'core/class.User',

            'group' =>
            'core/class.Group',

            'currentuser' =>
            'core/class.CurrentUser',

            'userpreferences' =>
            'core/class.UserPreferences',

            'registry' =>
            'core/class.Registry',

            'templateregistry' =>
            'core/class.TemplateRegistry',

            'templateprocessor' =>
            'core/class.TemplateProcessor',

            'templateloader' =>
            'core/class.TemplateLoader',

            'runtimecontext' =>
            'core/class.RuntimeContext',

            'persistentobservable' =>
            'core/class.PersistentObservable',

            'mediaitem' =>
            'core/class.MediaItem',

            'mediacontainer' =>
            'core/class.MediaContainer',

            'commentitem' =>
            'core/class.CommentItem',

            'mailmanager' =>
            'core/class.MailManager',

            // ------------------------------------------------------------

            'abstractuserdao' =>
            'dao/class.AbstractUserDAO',

            'daofactory' =>
            'dao/class.DAOFactory',

            'documentdao' =>
            'dao/class.DocumentDAO',

            'storagefactory' =>
            'dao/class.StorageFactory',

            'storagemysql' =>
            'dao/class.StorageMySQL',

            'userdaomysql' =>
            'dao/class.UserDAOMySQL',

            'mediadao' =>
            'dao/class.MediaDAO',

            'commentdao' =>
            'dao/class.CommentDAO',

            'referrerdao' =>
            'dao/class.ReferrerDAO',

            // ------------------------------------------------------------

            'searchcriteria' =>
            'dao/class.SearchCriteria',

            'abstractexpression' =>
            'dao/expression/class.AbstractExpression',

            'abstractjunction' =>
            'dao/expression/class.AbstractJunction',

            'abstractpropertyexpression' =>
            'dao/expression/class.AbstractPropertyExpression',

            'abstractsimpleexpression' =>
            'dao/expression/class.AbstractSimpleExpression',

            'conjunction' =>
            'dao/expression/class.Conjunction',

            'disjunction' =>
            'dao/expression/class.Disjunction',

            'eqexpression' =>
            'dao/expression/class.EqExpression',

            'noteqexpression' =>
            'dao/expression/class.NotEqExpression',

            'eqpropertyexpression' =>
            'dao/expression/class.EqPropertyExpression',

            'noteqpropertyexpression' =>
            'dao/expression/class.NotEqPropertyExpression',

            'geexpression' =>
            'dao/expression/class.GeExpression',

            'gepropertyexpression' =>
            'dao/expression/class.GePropertyExpression',

            'gtexpression' =>
            'dao/expression/class.GtExpression',

            'gtpropertyexpression' =>
            'dao/expression/class.GtPropertyExpression',

            'inexpression' =>
            'dao/expression/class.InExpression',

            'likeexpression' =>
            'dao/expression/class.LikeExpression',

            'leexpression' =>
            'dao/expression/class.LeExpression',

            'lepropertyexpression' =>
            'dao/expression/class.LePropertyExpression',

            'logicandexpression' =>
            'dao/expression/class.LogicAndExpression',

            'logicandpropertyexpression' =>
            'dao/expression/class.LogicAndPropertyExpression',

            'logicorexpression' =>
            'dao/expression/class.LogicOrExpression',

            'logicorpropertyexpression' =>
            'dao/expression/class.LogicOrPropertyExpression',

            'ltexpression' =>
            'dao/expression/class.class.LtExpression',

            'ltpropertyexpression' =>
            'dao/expression/class.LtPropertyExpression',

            'notlikeexpression' =>
            'dao/expression/class.NotLikeExpression',

            'notnullexpression' =>
            'dao/expression/class.NotNullExpression',

            'nullexpression' =>
            'dao/expression/class.NullExpression',

            // ------------------------------------------------------------

            'abstractoutputstream' =>
            'io/class.AbstractOutputStream',

            'fileoutputstream' =>
            'io/class.FileOutputStream',

            'abstractinputstream' =>
            'io/class.AbstractInputStream',

            'fileinputstream' =>
            'io/class.FileInputStream',

            // ------------------------------------------------------------

            'imagemanipulator' =>
            'lib/class.ImageManipulator',

            // ------------------------------------------------------------

            'config' =>
            'parse/class.Config',
            
            'wikiparser' =>
            'parse/class.WikiParser',

            'cowikiparser' =>
            'parse/class.CoWikiParser',

            'wikireverseparser' =>
            'parse/class.WikiReverseParser',

            'cowikireverseparser' =>
            'parse/class.CoWikiReverseParser',

            'xml' =>
            'parse/class.Xml',

            'abstractxmleventparser' =>
            'parse/class.AbstractXmlEventParser',

            'xmleventparser' =>
            'parse/class.XmlEventParser',

            'xmlobjectgraph' =>
            'parse/class.XmlObjectGraph',

            'xmlnode' =>
            'parse/class.XmlNode',

            'rssmanager' =>
            'parse/class.RssManager',

            'rssfeed' =>
            'parse/class.RssFeed',

            'rssitem' =>
            'parse/class.RssItem',

            // ------------------------------------------------------------

            'abstractplugin' =>
            'plugin/class.AbstractPlugin',

            'pluginloader' =>
            'plugin/class.PluginLoader',

            // ------------------------------------------------------------

            'fronthtmltransformer' =>
            'render/class.FrontHtmlTransformer',

            'layouter' =>
            'render/class.Layouter',

            'printhtmltransformer' =>
            'render/class.PrintHtmlTransformer',

            'xmlprettyhtmlprinter' =>
            'render/class.XmlPrettyHtmlPrinter',

            'lister' =>
            'render/class.Lister',

            'listerrow' =>
            'render/class.ListerRow',

            'listercolumn' =>
            'render/class.ListerColumn',

            'simplewindow' =>
            'render/class.SimpleWindow',

            // ------------------------------------------------------------

            'collection' =>
            'util/interface.Collection',

            'iterator' =>
            'util/interface.Iterator',

            'observer' =>
            'util/interface.Observer',

            'vector' =>
            'util/class.Vector',

            'vectoriterator' =>
            'util/class.VectorIterator',

            'observable' =>
            'util/class.Observable',

            'environment' =>
            'util/class.Environment',

            'uriinfo' =>
            'util/class.UriInfo',

            'httprequest' =>
            'util/class.HttpRequest',

            'httpresponse' =>
            'util/class.HttpResponse',

        );

        include_once 'class/' . $aPath[$sClass] . '.php';
    }

?>
