<?php

/**
 *
 * coWiki - includes/cowiki/core.function.php
 *
 * $Id: core.function.php,v 1.19 2003/10/23 12:58:00 dtg Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <cowiki-license@develnet.org> so we
 * can mail you a copy immediately.
 *
 * @package    core
 * @access     public
 *
 * @author     Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright  (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license    http://www.gnu.org/licenses/gpl.html
 * @version    $Revision: 1.19 $
 *
 */

/**
 * For debugging purposes 
 *
 * @package  core
 * @access   public
 *
 * @author   Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since    coWiki 0.3.0
 *
 * @todo     [D11N]  Complete documentation
 */
    set_error_handler('errorLogHandler');

    /**
     * Error log handler
     *
     * @access  public
     * @param   string
     * @param   string
     * @param   string
     * @param   string
     * @return  boolean
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function errorLogHandler($sErrNo, $sErrStr, $sErrFile, $sErrLine) {
        if (substr($sErrStr, 0, 4) == 'stat') {
            return true;
        }

        $sErr = '[' . date('D M j G:i:s Y') . ']'
                .' [error]'
                .' [client '.$_SERVER['REMOTE_ADDR'].'] ';

        $sErr .= '('.COWIKI_FULL_NAME.')'
                 .' '.$sErrStr
                 .' in file '.$sErrFile
                 .' line '.$sErrLine;

        @error_log($sErr);
    }

    $COWIKI_PROFILING_TIME = getMicroTime();
    $COWIKI_PROFILING_START_TIME = $COWIKI_PROFILING_TIME;
    $COWIKI_PROFILING_COUNTER    = 0;

    /**
     * Profiling functions
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check return type
     */
    function getMicroTime() {
        list($usec, $sec) = explode(' ', microtime());
        return ((float)$usec + (float)$sec);
    }

    /**
     * Profile
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function profile() {
        $GLOBALS['COWIKI_PROFILING_COUNTER']++;

        $nTime  = $GLOBALS['COWIKI_PROFILING_TIME'];
        $nCount = $GLOBALS['COWIKI_PROFILING_COUNTER'];

        echo    '<br />';
        echo    '<tt>';
        echo        '<span style="color:#880000">';
        echo            'Profilepoint&nbsp;#'
                        .$GLOBALS['COWIKI_PROFILING_COUNTER']
                        .sprintf(':&nbsp;%.3f', getMicroTime() - $nTime)
                        .'&nbsp;sec.';
        echo        '</span>';
        echo    '</tt>';
        echo    '<br />';

        $GLOBALS['COWIKI_PROFILING_TIME'] = getMicroTime();

    } // of profile()

    /**
     * We do not use PHP's dirname() function, it has strange side effects
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check return type
     */
    function getDirName($sPath) {
        $sDeli = substr(PHP_OS, 0, 3) == 'WIN' ? '\\' : '/';
        return substr($sPath, 0, strrpos($sPath, $sDeli)) . $sDeli;
    }

    /**
     * Get request base path
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function getRequestBasePath($sPath) {
        return substr($sPath, 0, strrpos($sPath, '/')) . '/';
    }

    /**
     * Newline2br
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function newline2br($sStr)  {
        return str_replace("\n", "<br />\n", $sStr);
    }

    /**
     * Cut off
     *
     * @access  public
     * @param   string
     * @param   integer
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nMaxLen"
     */
    function cutOff($sStr, $nMaxLen, $sSuffix = '...')    {
        if (strlen($sStr) > $nMaxLen) {
            $sStr = substr($sStr, 0 , $nMaxLen) . $sSuffix;
        }
        return $sStr;
    }

    /**
     * Cut off at start
     *
     * @access  public
     * @param   string
     * @param   integer
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nMaxLen"
     */
    function cutOffAtStart($sStr, $nMaxLen, $sPrefix = '...')    {
        if (strlen($sStr) > $nMaxLen) {
            $sStr =  $sPrefix . substr($sStr, -$nMaxLen);
        }
        return $sStr;
    }

    /**
     * Cut off word
     *
     * @access  public
     * @param   string
     * @param   integer
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nMaxLen"
     */
    function cutOffWord($sStr, $nMaxLen, $sSuffix = '...') {
        if (strlen($sStr) > $nMaxLen) {
            // Do not use ";" as delimiter, as it would destroy HTML
            // entities under unfortunate circumstances. 
            $aDelimiter = array(' ', '.', ',', '!', '?', '-', ':', '_', '/');
            $sStr = substr($sStr, 0, $nMaxLen + 1);

            $aPos = array();

            for ($i=0, $n=sizeof($aDelimiter); $i<$n; $i++) {
                $nPos = strrpos($sStr, $aDelimiter[$i]);
                if ($nPos) {
                    $aPos[] = $nPos;
                }
            }

            if (sizeof($aPos) > 0) {
                rsort($aPos);
                $sStr = substr($sStr, 0, $aPos[0]);
            }

            $sStr .= $sSuffix;
        }
        return $sStr;
    }

    /**
     * Wiki word
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function wikiWord($sStr) {
        static $aTrans = array(
            '' => 'A',  '' => 'A',  '' => 'A',  '' => 'A',  '' => 'Ae',
            '' => 'A',  '' => 'Ae', '' => 'C',  '' => 'E',  '' => 'E',
            '' => 'E',  '' => 'E',  '' => 'I',  '' => 'I',  '' => 'I',
            '' => 'I',  '' => 'N',  '' => 'O',  '' => 'O',  '' => 'O',
            '' => 'O',  '' => 'Oe', '' => 'O',  '' => 'U',  '' => 'U',
            '' => 'U',  '' => 'Ue', '' => 'Y',
            '' => 'ss', '' => 'a',  '' => 'a',  '' => 'a',  '' => 'a',
            '' => 'ae', '' => 'a',  '' => 'ae', '' => 'c',  '' => 'e',
            '' => 'e',  '' => 'e',  '' => 'e',  '' => 'i',  '' => 'i',
            '' => 'i',  '' => 'i',  '' => 'n',  '' => 'o',  '' => 'o',
            '' => 'o',  '' => 'o',  '' => 'oe', '' => 'o',  '' => 'u',
            '' => 'u',  '' => 'u',  '' => 'ue', '' => 'y',  '' => 'y'
        );
        $sStr = strtr($sStr, $aTrans);

        // Get rid of quotation mark
        $sStr = str_replace("'", '', $sStr);

        // Mark delimiters
        $sStr = preg_replace(
                    "#[\x20-\x2F\x3A-\x40\x5B-\x60\x7B-\x7F]#",
                    "\x01",
                    $sStr
                );

        // Get rid of special chars
        $sStr = preg_replace("#[^a-zA-Z0-9\x01]#", "", $sStr);

        // Recover delimiters as spaces
        $sStr = str_replace("\x01", " ", $sStr);

        // Capitalize the first character of each word
        return str_replace(" ", "", ucwords($sStr));
    }

    /**
     * Obfuscate email
     *
     * @access  public
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function obfuscateEmail($sStr) {

        $sStr = preg_replace(
            '#(.+)@(.+)\.([a-zA-Z]{2,})#US',
            '\1 (at) \2 (dot) \3',
            $sStr
        );

        return $sStr;
    }

    /**
     * Escape
     *
     * @access  public
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function escape($sStr) {
        $sStr = htmlspecialchars($sStr);
        return $sStr;
    }

    /**
     * Unescape
     *
     * @access  public
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function unescape($sStr) {
        $sStr = strtr($sStr,
                   array_flip(
                      get_html_translation_table(HTML_SPECIALCHARS)
                   )
                );
        return $sStr;
    }

    /**
     * Escape backslashes
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function escapeBackslashes($sStr) {
        return str_replace('\\', '\\\\', $sStr);
    }

    /**
     * Unescape backslashes
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function unescapeBackslashes($sStr) {
        return str_replace('\\\\', '\\', $sStr);
    }

    /**
     * Expects warp=physical|hard
     *
     * @access  public
     * @param   string
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check the parameter type of "$nLen"
     * @todo    [D11N]  Check return type
     */
    function quote($sStr, $nLen = 78) {
        $aTok = explode("\n", $sStr);
        
        for ($i=0, $n=sizeof($aTok); $i<$n; $i++) {

            if (strlen($aTok[$i]) > $nLen) {
                // Already quoted?
                preg_match('#^(>*)#', $aTok[$i], $aMatches);
                $aTok[$i] = '>' . $aTok[$i];
                $aTok[$i] = wordwrap($aTok[$i], $nLen);
                $aTok[$i] = str_replace("\n", "\n>".$aMatches[1], $aTok[$i]);
            } else {
                $aTok[$i] = '>' . $aTok[$i];
            }
        }

        $sStr = join("\n", $aTok);
        
        // Remove empty quoted lines
        return preg_replace('#^>+$#m', '', $sStr);
    }
  

    /**
     * Set catalog
     *
     * @access  public
     * @param   string
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function setCatalog($sPathLocale, $sName)    {
        if ($sPathLocale[strlen($sPathLocale)-1] != '/') {
            $sPathLocale .= '';
        }

        if (@include($sPathLocale . $sName. '.cat')) {
            foreach ($__ as $k => $v) {
                $GLOBALS['__'][$k] = $v;
            }
        }
    }

    /**
     * __
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    function __($sMsgId) {
        if (isset($GLOBALS['__'][$sMsgId])) {
            return $GLOBALS['__'][$sMsgId];
        }

        return $sMsgId;
    }

    /**
     * Get salt
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function getSalt() {
        return getSaltChar().getSaltChar();
    }

    /**
     * Get salt char
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function getSaltChar(){
        do ($nNum = randomSaltNum());
        while (($nNum > 57 && $nNum < 65) || ($nNum > 90 && $nNum < 97));
        return chr($nNum);
    }

    /**
     * Random salt num
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    function randomSaltNum()    {
        mt_srand((double)microtime()*1000000);
        return mt_rand(46, 122);
    }

    /**
     * Use with caution on self referenced structures! Watch your httpd!
     *
     * @access  public
     * @param   object
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check the parameter type of "$mVar"
     */
    function printr($mVar) {
        ob_start();
        print_r($mVar);
        $sContent = ob_get_contents();
        ob_end_clean();

        echo '<pre>';
        echo htmlentities($sContent);
        echo '</pre><hr />';
    }

?>
